#ifndef AT_CMD_SUPPORT_H_INCLUDED
#define AT_CMD_SUPPORT_H_INCLUDED

#include "stm32l1xx.h"
#include "uart_api.h"
#include "stdio.h"
#include "string.h"

/**
    @ingroup device_drivers
    @defgroup at_cmd_support      AT-
    @{
*/

 /**
    @file at_cmd_support.h
    @author RedBees, LLC
    @copyright This file is distributed under the terms of the MIT license.
 */

///   ( ),        .
/// ,     SARA-R410-02B   1024 .
///          .
#define AT_RESP_BUFFER_LENGTH       1024

//   .
//   : <> CR CR LF <> [OK | ERROR] CR LF
// ,  , -    "OK"  "ERROR"   ;
//    .  DEBUG_NO_RESP_END_SEARCH   .   
//AT_ReadResponseBuffer()    ,   ,   .
//#define DEBUG_NO_RESP_END_SEARCH

#ifndef NULL
#define NULL ((void *)0)
#endif

///. AT_CreateUDPSocket()
#define INVALID_SOCKET_ID       255

typedef enum {
    /**      */
    AT_NO_RESPONSE,
    /**       */
    AT_ERROR,
    /**      -;  ,   */
    AT_FORMAT_ERROR,
    /**    UDP-:  ,   ,     ; ,     */
    AT_LENGTH_DOES_NOT_MATCH,
    /**   ,   */
    AT_NO_ERROR
} at_udp_error_t;

///1 -    URC     .       NIDD.
///  ,    ,     URC,   UDP.
#define USE_ADVANCED_URC_SEARCH         1

/// URC,    
#define NUM_URC_SUPPORTED               2

///   URC
#define URC_NAME_LENGTH                 16
///,  URC-,          
typedef struct {

    /**   URC,       */
    uint8_t URC_string_name[URC_NAME_LENGTH];
    /**   ,     URC */
    uint8_t match_counter;
    /** ,    ,    URC (     URC) */
    uint8_t URC_header_detected;
    /** ,    ,  URC  ,       */
    uint8_t URC_detected;

} expected_URC_entry_t;

/**
    @brief   .

         ,  .     .
          ,      
    -.       .

    @param [out] buffer_out   ,       .
    @param [in] buffer_length  ,     
    @warning      ,    
    @param [out] is_overflowed     ,      RESP_BUFFER_LENGTH    . 
        ,      NULL.
    @param [out] overrun_detected     ,           
    ,         .      ,     
    NULL.
    @param [out] error_status   1,      ;     - 0.    , 
        NULL.
    @param [in] max_wait_time       ( ).        ,  ,  .
    @return   .
*/
uint16_t AT_ReadReponseBuffer(uint8_t* buffer_out,uint16_t buffer_length,uint8_t* is_overflowed,uint8_t* overrun_detected,uint8_t *error_status,uint32_t max_wait_time);

/**
    @brief  AT-  .

    @warning AT-    NULL-  ( C).
*/
void AT_SendCommand(const uint8_t *cmd);

/**
    @brief  UDP-  .

    @param [in] max_wait_time     ,  .
    @return   .     ,       INVALID_SOCKET_ID.
*/
uint8_t AT_CreateUDPSocket(uint32_t max_wait_time);

/**
    @brief     UDP.

    @param [in] socket_id  
    @param [in] max_wait_time      

    @return    - 1,      - 0
*/
uint8_t AT_CloseUDPSocket(uint8_t socket_id,uint32_t max_wait_time);

/**
    @brief   UDP   .

    @param [in] socket_id  ,      
    @param [in] target_IP_string IP-     ,  "127.0.0.1"
    @param [in] target_port  ,     
    @param [in] data     
    @param [in] data_length   
    @param [in] max_wait_time     
    @return   ,   at_udp_error_t
*/
at_udp_error_t AT_SendUDPData(uint8_t socket_id,uint8_t *target_IP_string,uint16_t target_port,uint8_t *data,uint16_t data_length,uint32_t max_wait_time);
/**
    @brief      dBm

    @param [in] timeout     
    @return     dBm.

    @warning      -111 dBm  -51 dBm. ,    ,    .
         ,    .  , -51 dBm  "-51 dBm  ", -111 dBm 
    "-111 dBm  ".
*/
int16_t AT_GetRSSI(uint32_t timeout);
/**
    @brief  ,       UDP-.

    @param [out] socket_id   ,      ,  ;    ,   NULL
    @param [out] packet_length   ,         ;    ,   NULL
    @return      ,      ,   0;    - 1
*/
uint8_t AT_CheckUDPReceived(uint8_t *socket_id,uint16_t *packet_length);
/**
    @brief     UDP-

    @param [in] socket_id  ,     
    @param [in] read_length  ,   
    @param [out] source_IP   ,      - ;     ,    NULL
    @param [out] source_port   ,      ,    ;    ,    NULL
    @param [out] data_out   ,     ;       read_length .    ,    NULL
    @param [in] max_wait_time       
    @return  ,   at_udp_error_t
*/
at_udp_error_t AT_ReadUDPData(uint8_t socket_id,uint16_t read_length,uint8_t *source_IP,uint16_t *source_port,uint8_t *data_out,uint32_t max_wait_time);

/**
    @brief   NIDD- ,   ,   

    @param [out] data_out   ,     
    @warning        513  ( 512       )
    @return          

    @note     ,          URC   ,
    @note . \link USE_ADVANCED_URC_SEARCH \endlink.       ,   .
*/
uint16_t AT_CheckNIDDReceived(uint8_t *data_out);

/**
    @brief  ICCID SIM-/

    @param [in] ICCID ,     ICCID **  **.      50 .
    @param [in] timeout      ,  
    @return   - 1,    - 0
*/
uint8_t AT_ReadICCID(uint8_t *ICCID,uint32_t timeout);

/**
    @brief  IMEI 

    @param [in] IMEI ,     IMEI **  **.      50 .
    @return   - 1,    - 0
*/
uint8_t AT_ReadIMEI(uint8_t *IMEI,uint32_t timeout);

/**
    @brief      NIDD

    @warning       

    @param [in] str ,   
    @warning    - 450 
    @param [in] timeout     
    @return    - 1,     - 0.
*/
uint8_t AT_SendStringUsingNIDD(uint8_t *str,uint32_t timeout);

/**
    @brief      LTE

    @param [in] timeout     
    @return      - 1,          - 0
*/
uint8_t AT_IsRegistered(uint32_t timeout);
/**
    @}
*/

/**
    @brief       PSM

    @param [in] state 1 -   PSM:      T3324 ( ); 0 -    .
    @return     ,    ;  ,   .
*/
uint8_t AT_SwitchPSM(uint8_t state,uint32_t timeout);

void __AT_FSM_rearm(void);

#endif /* AT_CMD_SUPPORT_H_INCLUDED */
